.text
.globl start
.code16
.org 0x0

# The processor will load this bootloader into 0x07C00, which
# will make 0x07C00 the segment, and the first statement is addressed
# at offset zero
			
jmp start		# Jumping to start will initialize CS to 0x07C0

.set BOOTSEG, 0x07C0	# Bootloader location
.set INITSEG, 0x9000	# New bootloader location
.set SYSSEG, 0x1000	# Load kernel from disk into this location
	
start:
	# Initialize the segment registers
	movw %cs, %ax
	movw %ax, %ds
	movw %ax, %es
	movw %ax, %fs
	movw %ax, %gs
	movw %ax, %ss

	# Hide the damn cursor
	movb $32, %ch
	movb $1, %ah
	int $0x10

	# Move bootloader to new location
	movw $BOOTSEG, %ax	# DS and ES gets used by movsb instruction
	movw %ax, %ds		# DS:SI -> ES:DI
	movw $INITSEG, %ax
	movw %ax, %es
	movw $256, %cx		# Move 256 Words == 512 Bytes
	xorw %si, %si		# Initialize these registers to zero
	xorw %di, %di
	rep movsw		# Repeat [cx] times
	ljmp $INITSEG, $loadK	# Go to loadK in our new segment location

# Load kernel from disk into memory
loadK:
	movw $0x00, %ax	# Reset floppy drive
	movb $0, %dl	# A = Drive 0
	int $0x13
	jc loadK	# Reset again if error.  This usually fails a couple times.
	
read:
	movw $0x1000, %ax	# ES:BX = 1000:0000
	movw %ax, %es
	movw $0, %bx

	movb $0x02, %ah	# Load disk data to ES:BX
	movb $32, %al	# Load 32 sectors (16KB): MAKE SURE THIS IS YOUR WHOLE KERNEL
	movb $0, %ch	# Cylinder=0
	movb $2, %cl	# Sector=2 : Begin at sector 2
	movb $0, %dh	# Head=0
	movb $0, %dl	# Drive=0
	int $0x13	# Read disk

	jc read


# Disable interrupts
# Move Kernel to 0x0 (beginning of physical mem)
	cli
	cld
	movw $SYSSEG, %ax	# Source
	movw %ax, %ds
	movw $0x00, %ax		# Destination
	movw %ax, %es
	movw $8192, %cx		# Move 16KB (8192 words) 0000h -> 2000h
	xorw %si, %si
	xorw %di, %di
	rep movsw		# Move words 4096 times
	
# Set up GDT and IDT
	movw %cs, %ax
	movw %ax, %ds
	
	lidt idt_48		# Load IDT register
	lgdt gdt_48		# Load GDT register

# Enable A20 line
	call empty_8042
	movb $0xD1, %al
	outb %al, $0x64		# Set output port of kb ctrlr
	call empty_8042 	# Wait for kbd to finish handling input
	movb $0xDF, %al
	outb %al, $0x60		# Set value of output port
	call empty_8042		# Wait for kbd to finish handling input

# Go into protected mode and jump to kernel
	movl %cr0, %eax
	orl $1, %eax
	movl %eax, %cr0

	ljmp $8, $0x0

#------------------------------------------------------------------
# This is our GDT, located at 0x9000:WhereEverThisIs
gdt:
	.quad 	0x0000000000000000	# Null Descriptor

	.word 0xFFFF	# 4GB Limit	# Ring0 CS
	.word 0x0000	# Base = 0
	.word 0x9A00	# Read/Exec
	.word 0x00CF	# Granularity=4096

	.word 0xFFFF	# 4GB Limit	# Ring0 DS
	.word 0x0000	# Base = 0
	.word 0x9200	# Read/Write
	.word 0x00CF	# Granularity=4096

# These values go into the GDT register
gdt_48:
	.word 0x38	# Limit 7 Entries = 56 Bytes
	.word gdt	# Base address of GDT
	.word 0x9

# These values go into the IDT register
idt_48:
	.word 0
	.long 0

# Wait for kbd to finish handling input
empty_8042:
	inb $0x64, %al
	testb $2, %al
	jnz empty_8042
	ret

# Boot sectors must be 512B in size and end with AA55h
.org 	0x1FE, 0x90	# Fill to 0x1FE with 0x90 (nop)
.word 	0xAA55
