/*
 * Function/Structure/Constant definitions
*/

#define SYSTEM_H 1

// Memory utilization
#define GDT_ADDR 	0x00090000	// GDT base address
#define GDT_LIMIT 	0x37		// Size of GDT (subtract 1 for limit value)
#define IDT_ADDR 	0x00090038	// IDT base address
#define IDT_LIMIT 	0x179		// Size of IDT
#define TSS_ADDR 	0x000901B8	// TSS base address
#define TSS_LIMIT 	0x67		// Size of TSS
#define TSS_GDT_ENTRY 	0x28		// The TSS GDT Descriptor index
#define PAGE_DIR 	0x00091000	// Page directory physical location
#define PAGE_TABLE_LH 	0x00092000	// Lower half Page table physical location
#define PAGE_TABLE_HH 	0x00096000	// Higher half Page table physical location

#define VIDEO_BUFFER	0x000B8000	// Video buffer start

#define HEAP_START 	0x00800000
#define STACK_START 	0x00FFFFFF

#define USER_START 	0x01000000	// User space starts here

#define MEM_SIZE 	0x02000000	// Amount of physical memory installed (32MB)
#define HIGH_HALF 	0x80000000	// Beginning of Kernel memory (2GB)   

#define CLOCK_RATE 	1		// PIC interrupt interval

/* 
 * Stack Frame - Interrupts support
*/
struct REGS
{
	unsigned int gs, fs, es, ds;				// Saved segments
	unsigned int edi, esi, ebp, esp, ebx, edx, ecx, eax;	// Saved register from pusha
	unsigned int int_no, err_code;				// our 'pushb #' and errorcodes
	unsigned int eip, cs, eflags, useresp, ss;		// Pushed by processor automatically
};

extern void flush_segments(void);
