/*
 * Initialize IRQ, IDT
*/
#include "asm.h"
#include "system.h"
#include "idt.h"
#include "timer.h"

/*
* Remap IRQ 0-15 to IDT 32-47
* We need to do this because by default IRQ 0-7 are vectored to IDT 8-15 and 
* IRQ 8-15 are vectored to IDT 112-120. IDT 8-15 are reserved for exceptions.
*/
void k_remap_irq(void)
{	
	// outb(value, port)
	outb(0x11, 0x20);
	outb(0x11, 0xA0);
	outb(0x20, 0x21);
	outb(0x28, 0xA1);

	outb(0x04, 0x21);
	outb(0x02, 0xA1);

	outb(0x01, 0x21);
	outb(0x01, 0xA1);
	outb(0x0, 0x21);
	outb(0x0, 0xA1);
}


/*
 * Install the IDT at 0x92000 and load the IDTR.
 * Will occupy 0x92000 to 0x92180
*/
void k_install_idt(void)
{
	// IDT starts at this address
	struct IDT * idt_dest = (struct IDT *) IDT_ADDR;
	// lidt will load this structure
	struct IDT_PTR idtr = {IDT_LIMIT, IDT_ADDR};

	// Interrupt gates are 0x8E, Trap gates are 0x8F
	// idt_entry_set_values( Selector, Offset, Flags, IDT)
	idt_entry_set_values(0x08, (unsigned)isr0, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)isr1, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)isr2, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)isr3, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)isr4, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)isr5, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)isr6, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)isr7, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)isr8, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)isr9, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)isr10, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)isr11, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)isr12, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)isr13, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)isr14, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)isr15, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)isr16, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)isr17, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)isr18, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)isr19, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)isr20, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)isr21, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)isr22, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)isr23, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)isr24, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)isr25, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)isr26, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)isr27, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)isr28, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)isr29, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)isr30, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)isr31, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)irq0, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)irq1, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)irq2, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)irq3, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)irq4, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)irq5, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)irq6, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)irq7, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)irq8, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)irq9, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)irq10, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)irq11, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)irq12, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)irq13, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)irq14, 0x8E, idt_dest++);
	idt_entry_set_values(0x08, (unsigned)irq15, 0x8E, idt_dest);

	set_idt_loc(&idtr);

	k_remap_irq();
	k_timer_install(CLOCK_RATE);

	sti();
}


/* Set the following values into the idt descriptor entry */
void idt_entry_set_values(	unsigned short selector, 
				unsigned int offset, 
				unsigned char flags, 
				struct IDT* e)
{
	e->selector 	= selector;
	e->offset_low 	= (offset & 0xFFFF);
	e->offset_high 	= (offset >> 16) & 0xFFFF;
	e->always0 	= 0;
	e->flags 	= flags;
}

