/*
 * Intall the TSS
*/
#include "asm.h"
#include "system.h"
#include "tss.h"

/*
 * The k_install_tss procedure will setup the global TSS.
 * The TSS is very important if we were doing hardware multitasking.  However,
 * since we're implementing our multitasking in software, the TSS is relegated
 * to facilitate Stack Switching.  The SS0 and ESP0 fields hold pointers to
 * the stack that will be used during RING0 to RING3 privilege level changes.
*/
void k_install_tss(void)
{
	struct TSS_T * tss = (struct TSS_T *) TSS_ADDR;
	unsigned short tssLoc = TSS_GDT_ENTRY;

	tss->backlink 	= 0x0;
	tss->esp0 	= 0x001FEFFF;	// A little bit lower than Ring0 ESP
	tss->ss0 	= 0x10;		// Using Ring0 SS
	tss->esp1 	= 0x0;
	tss->ss1 	= 0x0;
	tss->esp2 	= 0x0;
	tss->ss2 	= 0x0;
	tss->cr3 	= 0x0; 
	tss->eip 	= 0x0;
	tss->eflags 	= 0x0;
	tss->eax 	= 0x0;
	tss->ecx 	= 0x0;
	tss->edx 	= 0x0;
	tss->ebx 	= 0x0;
	tss->esp 	= 0x0;
	tss->ebp 	= 0x0;
	tss->esi 	= 0x0;
	tss->edi 	= 0x0;
	tss->es 	= 0x10;
	tss->cs 	= 0x08;
	tss->ss 	= 0x10;
	tss->ds 	= 0x10;
	tss->fs 	= 0x10;
	tss->gs 	= 0x10;
	tss->ldt 	= 0x0;
	tss->trace 	= 0x0;
	tss->bitmap 	= 0x0;

	set_tr(tssLoc);
}

